/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package Simulation;

import java.util.ArrayList;
import java.util.EnumMap;
import java.util.List;
import java.util.Map;
import ws3dproxy.model.WorldPoint;
import ws3dproxy.util.Constants;

/**
 *
 * @author Danilo
 */
public abstract class SimulationCreature
{
    public enum StakeholderType
    {
        CREATURE
    }

    private StakeholderType entityType;
    private WorldPoint position;
    private EnumMap<SensorType,ISimulationSensor> sensorHandler = new EnumMap<SensorType, ISimulationSensor>(SensorType.class);

    public SimulationCreature(StakeholderType _entityType,WorldPoint point)
    {
        entityType = _entityType;
        position = point;
    }

    /**
     * @return the entityType
     */
    public StakeholderType getEntityType()
    {
        return entityType;
    }

    /**
     * @param entityType the entityType to set
     */
    public void setStackHolderType(StakeholderType entityType)
    {
        this.entityType = entityType;
    }

          /**
     * @return the position
     */
    public WorldPoint getPosition()
    {
        return position;
    }

    /**
     * @param position the position to set
     */
    public void setPosition(WorldPoint _position)
    {
        this.position = _position;
    }

    /***
     * Set Sensors available in stackholder
     * @param List of sensors
     */
    public void addSensorsAvailable(ISimulationSensor sensorHandlerClass)
    {
        if (sensorHandlerClass != null)
        {
            sensorHandler.put(sensorHandlerClass.getSensorType(), sensorHandlerClass);
        }
        else
        {
            throw new IllegalArgumentException("Sensor Handler Class can not be null");
        }
    }

    /***
     * Get Sensors available in stackholder
     * @return
     */
    public List<SensorType> getSensorsAvailable()
    {
        List<SensorType> sensorTypeList = null;

        if (sensorHandler.size() > 0)
        {
           sensorTypeList = new ArrayList<SensorType>();

           for(Map.Entry<SensorType,ISimulationSensor> entry : sensorHandler.entrySet())
           {
                sensorTypeList.add(entry.getKey());
           }
        }

        return sensorTypeList;
    }

    /***
     * Get Count Sensors available in stackholder
     * @return
     */
    public Integer getCountSensorsAvailable()
    {
        return sensorHandler.size();
    }

     /***
     * Get Sensors enabled in stackholder
     * @return
     */
    public List<SensorType> getSensorsEnabled()
    {
        List<SensorType> sensorTypeList = null;

        if (sensorHandler.size() > 0)
        {
           sensorTypeList = new ArrayList<SensorType>();

           for(Map.Entry<SensorType,ISimulationSensor> entry : sensorHandler.entrySet())
           {
               if (entry.getValue().isSensorEnabled())
               {
                    sensorTypeList.add(entry.getKey());
               }
           }
        }

        return sensorTypeList;
    }

     /***
     * Get Sensors enabled count in stackholder
     * @return
     */
    public Integer getCountSensorsEnabled()
    {
        Integer count = 0;

        if (sensorHandler.size() > 0)
        {           
           for(Map.Entry<SensorType,ISimulationSensor> entry : sensorHandler.entrySet())
           {
               if (entry.getValue().isSensorEnabled())
               {
                    count++;
               }
           }
        }

        return count;
    }

    public ISimulationSensor getSensorHandler(SensorType sensorType)
    {
        return sensorHandler.get(sensorType);
    }

     /***
     * Convert the object type to Simulation Environment Objects
     * @return The entity Type detected by the sensor
     */
    public static StakeholderType getPrimaryTypeFromThingCategory(int categoryType)
    {
        StakeholderType entityValue = null;

        switch (categoryType)
        {
            case Constants.categoryCREATURE:
                entityValue = StakeholderType.CREATURE;
                break;
        }

        return entityValue;
    }
}
